%VIDSYNC will syncronize some formats of video data (currently mostly only 
% AVI) with some other time stamped set of data (likely force plate data).  
% The user can select the 2 video and data files and also which frames to
% syncronize from using the VIDSYNC_VIDGUI.M gui script .
% This program will create a MATLAB movie file (exported AVI) that that 
% displays the video and other data set together.  This file has a standard
% name and is time dated.
% 04/29/05 - A. McClung - Stanford University (1st stable version)

%Clean up the workspace - either do this at start or cleanup at end
clear;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Variables that may need to be changed with video & data properties
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%NOTE: Some of these initial variables are set in both VIDSYNC_Rxx.M and VIDSYNC_VIDGUI.M, so be sure to change both.
datasamp=1000;                  % The default force plate data sampling rate is 1kHz (1000Hz)
framerate=30;                   % The frame rate for the exported video (Matlab default is 15fps)
quality=75;                     % The quality of the exported video (Matlab dfault is 75)
codec='Indeo5';                 % The compression method for the exported video (Matlab default is 'Indeo5')
datacols=[4 5 6];               % Columns containing data for plotting
syncplotzero=1;                 % Start the syncronized result at zero or actual time from input files
                                % Set to '1', to plot relatively from t=0
                                % Clear to '0', to plot in time relative to input video/data
axtext='XYZ';                   % Labels for data axes
axType='Accel';                 % Type of data
axYUnits='[V]';                 % Units for Y axis
axcolor='bgrcmk';               % Colors for data plots
maxvideoframes=200;             % "Chunk size" - Maximum number of video frames loaded into memory at any time (600+ often causes virtual memory errors)

%Get the video data file location
[vfile,vpath]=uigetfile('*.avi','Select the video file');
%Get the additional (ie. force plate) data file location
[afile,apath]=uigetfile('*.dat;*.mat','Select the data file');

%Determine the frame rate and length
movinfo=aviinfo([vpath vfile]);
movlength=movinfo.NumFrames/movinfo.FramesPerSecond;
fprintf('Video file length: %1.2f secs\n', movlength);
fprintf('Total frames: %d\n', movinfo.NumFrames);
%Load the video file (OR Just load 1st chunk if video is too long)
if (movinfo.NumFrames>maxvideoframes)
    frames2load=[1:maxvideoframes];
else
    frames2load=[1:movinfo.NumFrames];
end
fprintf('Loading frames [1:%d] to sync ....\n',frames2load(size(frames2load,2)));
mov2sync=MYaviread([vpath vfile],frames2load);
fprintf('DONE!\n');

%Load the data file
fprintf('Reading the data file...');
data2sync=load([apath afile]);
fprintf('\nDONE!\n');
%Determine if the data is in a struct, if so extract data, time and sampling rate
if isstruct(data2sync)==1
    structdata=1;               % Marker for data in struct form
    fprintf('NOTE: Struct data detected!\nNOTE: Please select variable and columns!\n')
    getstructdata;              % Script to extract the data from struct
end
% Determine the length of the data
datanum=size(data2sync,1);
datalength=datanum/datasamp;
fprintf('Data file length: %1.2f secs\n', datalength);

%Syncronize the video with the additional data
%  NOTE: Usually the data sampling rate will be higher than the video frame
%   rate, so the data will be synced to the video by default
%  NOTE: If the video is sampled faster than data, an ELSE statement can be
%   added to the followinig statement to handle that case.
if datasamp>=movinfo.FramesPerSecond
    %Select the first frame from video & data to start syncronizing.  The 
    % two frames are returned from the VIDSYNC_VIDGUI script.
    fprintf('Starting syncronizer GUI...\n')
    %Get the 1st VIDEO and DATA frames to sync from gui scrpt
    [movframe1 dataframe1]=vidsync_vidgui(mov2sync,data2sync,datasamp);
    %Determine how much time from each (video & data) can be syncronized
    timemarks(1,:)=[movframe1 movinfo.NumFrames]/movinfo.FramesPerSecond;
    timemarks(2,:)=[dataframe1 datanum]/datasamp;
    synctime=min(timemarks(:,2)-timemarks(:,1));    % Time length of sync
    if (timemarks(1,2)-timemarks(1,1))>(timemarks(2,2)-timemarks(2,1))
        %Video is longer than data
        fprintf('The VIDEO footage is LONGER than the acquired DATA!\n')
        timemarks(1,2)=timemarks(1,1)+synctime;
    else
        %Data is longer than video
        fprintf('The acquired DATA is LONGER than the VIDEO footage!\n')
        timemarks(2,2)=timemarks(2,1)+synctime;
    end
    %Display the video and data that will be used
    fprintf(['The video and data can be syncronized for ' num2str(synctime) ' secs!\n']);
    fprintf(['Video [Start : Stop] - TIME(s)->[ %1.4f : %1.4f ] - FRAMES->[%6d : %6d ]\n'],...
        timemarks(1,1),timemarks(1,2),movframe1,round(timemarks(1,2)*movinfo.FramesPerSecond))
    fprintf(['Data  [Start : Stop] - TIME(s)->[ %1.4f : %1.4f ] - FRAMES->[%6d : %6d ]\n'],...
        timemarks(2,1),timemarks(2,2),dataframe1,round(timemarks(2,2)*datasamp))
    fprintf('Note: Times and frames are based on original input files.\n');
    %Build synced data and remove excess (RELATIVE TO VIDEO TIMING)
    syncnum=round(timemarks(1,2)*movinfo.FramesPerSecond)-movframe1;    %Number of video frames to syncronize
    for i=1:syncnum
        %Get the video time relative to specified 1st frame
        %NOTE: EITHER '-1' or '-0' depending on where you want video to start.
        %  '-1' starts at zero, '-0' starts at 1st frame time
        synctime(i)=(i-1)/movinfo.FramesPerSecond;
        %Use the video timing to get the related data frame
        % -This the closest matching data sample for video, shifted by the specified 1st data frame
        syncdataint(i)=dataframe1+round(synctime(i)*datasamp);
        %Store the data value that is now synced to the video
        %NOTE: Get the desired data - forces are cols 4-6 for force plate at SU
        syncdata(i,:)=data2sync(syncdataint(i),datacols);          
    end
    if syncplotzero==1
        % Plot relatively from t=0
        syncoffset=0;
        smoothtime=([syncdataint(1):syncdataint(syncnum)]-dataframe1)/datasamp;
    else
        % Plot in time relative to input video/data
        syncoffset=dataframe1/datasamp;
        smoothtime=[dataframe1:syncdataint(syncnum)]/datasamp;
    end
    synctime=synctime + syncoffset;
end

%%% - LETS START MAKING A MATLAB MOVIE - %%%
%NOTE: The movie is generated frame by frame on the screen, and the present 
% figure is captured at each stage and added to the movie

%Get the current time, to append to filename
curtime=clock;
%Add the date...
suffix=['_' num2str(curtime(1)-2000,'%02d') num2str(curtime(2),'%02d') num2str(curtime(3),'%02d')];
%Now, add the time
suffix=[suffix '-' num2str(curtime(4),'%02d') num2str(curtime(5),'%02d') num2str(floor(curtime(6)),'%02d')];

%%Create the movie framing
h=figure;
set(h,'Position',[10 50 500 800]);          % Screen size and placement while the movie is being generated
M=avifile(['VidDataSync' suffix '.avi']);   % Open a movie file for writing
M.Fps=framerate;                            % Set the frame rate
M.Quality=quality;                          % Set the movie quality
M.Compression=codec;                        % Set the compression method

%Draw the initial figure
%NOTE: May need to adjust array size for different size video frames/data plots
arraysize=size(datacols,2)+3;       % Let video take up 3 subplots
h1a=subplot(arraysize,1,1);
h1=image(mov2sync(1).cdata);
title('Syncronized Video & Data','FontSize',14)
axis image off;
%Try to resize the image to appropriate portions
%SEE BELOW
for i=4:arraysize
    eval(['h' num2str(i) '=subplot(' num2str(arraysize) ',1,' num2str(i) ');']);
    %Plot the real data in subplot
    eval(['h' num2str(i) '=plot(smoothtime,data2sync(syncdataint(1):syncdataint(syncnum),' num2str(datacols(i-3)) '),''' axcolor(i-3) ''');'])
    axis tight; hold on;
    eval(['title(''' axtext(i-3) '-axis ' axType ' data'')'])
    grid on;
    %Add the X-axis label for the last plot
    if i==arraysize
        xlabel('Time [sec]');
    end
    ylabel([axType ' ' axYUnits]);
    %Determine axes for data framing
    curaxis=axis;
    eval(['h' num2str(i) 'b=plot([0 0]+syncoffset,[-3e3 3e3],''k'',''LineWidth'',3);'])
    axis(curaxis);
end

%Resize and reposition the video plot
%NOTE: This may need to occur after the other subplots are created - it was not happy above
%NOTE: May need to adjust these values for different size video frames and number of data columns
% Format - [lower left corner X, lower left corner Y, width, height]
%See 'help subplot' for info on 'Position' variable
set(h1a,'Position',[.05 .3 .9 .9])


% Loop thru and add a chunk of video (frames) to te movie each time
for i=1:((movframe1+syncnum-1)/maxvideoframes)+1
    %Set the frames that will be loaded for addition to the video
    frames2load=[((i-1)*maxvideoframes+1):(i*maxvideoframes)];
    %Get the correct number of frames for the first and last chunks
    if i==1
        frames2load=[movframe1:maxvideoframes];
    end
    if (i==floor((movframe1+syncnum-1)/maxvideoframes)+1)
        frames2load=[frames2load(1):movframe1+syncnum-1];
    end
    %Load the necessary video chunk (except in the 1st pass)
    if frames2load(1)~=movframe1
        clear mov2sync;
        fprintf('Loading frames [%d:%d] ....\n',frames2load(1),frames2load(size(frames2load,2)));
        mov2sync=MYaviread([vpath vfile],frames2load);
        fprintf('\nDONE!\n');
    end
    %LOOP the following to add each sync'd video & data chunk to movie
    fprintf('Syncronizing frames [%d:%d]\n',frames2load(1),frames2load(size(frames2load,2)));
    for j=frames2load
        %%Display the image
        set(h1,'CData',mov2sync(j-(i-1)*maxvideoframes).cdata)
        %%Display the additional data
        for k=4:arraysize
            eval(['set(h' num2str(k) 'b,''XData'',[1 1]*(synctime(j-movframe1+1)))'])
        end
        %%Add the frame to the movie - Based on 'help avifile'
        M=addframe(M,getframe(h));
        %Give the user some feedback
        if (mod(j,10)==0)
            fprintf([' ' num2str(j)]);
        end
        if (mod(j,100)==0)
            fprintf('\n');
        end
    end
end
    
%Finalize the movie file
M=close(M);

%Close the figure window
close(h);